import 'dart:convert';

import 'package:gemini_protocol/gemini_protocol.dart';
import 'package:test/test.dart';

bool _listEqual<T>(List<T> first, List<T> second) {
  if (first.length != second.length) return false;

  for (int i = 0; i < first.length; i++) {
    if (first[i] != second[i]) return false;
  }

  return true;
}

void main() {
  group('Request', () {
    test('Decode byte stream', () async {
      final stream = Stream.fromIterable([
        utf8.encode(
          "gemini://example.com\r\nblahblahblah_it_should_be_ignored",
        ),
      ]);
      final request = await Request.fromByteStream(stream);

      final expectedUrl = Uri.parse("gemini://example.com");

      expect(request.url == expectedUrl, isTrue);
    });

    test('encode', () {
      final request = Request(url: Uri.parse("gemini://example.com"));

      final actual = request.encode();
      final expected = utf8.encode("gemini://example.com\r\n");

      expect(_listEqual(actual, expected), isTrue);
    });
  });

  group('Response', () {
    test('Decode byte stream', () async {
      final stream = Stream.fromIterable([
        utf8.encode("20 text/gemini\r\nSome body"),
      ]);
      final response = await Response.fromByteStream(stream);

      final expectedStatusCode = 20;
      final expectedMeta = "text/gemini";
      final expectedBody = utf8.encode("Some body");

      expect(response.statusCode == expectedStatusCode, isTrue);
      expect(response.meta == expectedMeta, isTrue);
      expect(_listEqual(response.body, expectedBody), isTrue);
    });

    test('Encode', () {
      final response = Response(
        statusCode: 20,
        meta: "text/gemini",
        body: utf8.encode("Some body"),
      );

      final actual = response.encode();
      final expected = utf8.encode("20 text/gemini\r\nSome body");

      expect(_listEqual(actual, expected), isTrue);
    });
  });
}
